/****************************************************************************
 AutoText.h
 
 Description:
    This API allows other applications on the pager to use the AutoText 
    functionality.  The AutoTextEdit object is designed so that it can be
    used instead of an Edit object, and AutoText will magically come to life.
 
 (C) 1998-1999 Research In Motion Ltd.
 ***************************************************************************/

#ifndef _AUTOTEXT_H_
#define _AUTOTEXT_H_

#include "Pager.h"
#include "UI32.h"
#include "Database.h"

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _AUTOTEXT_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif

//Add an AutoText entry to the database.
DbRecordHandle
DLLAccess 
AddAutoText( char const * const  pOriginalText, 
            char const * const  pReplacementText );


//A class that envelops the AutoText functionality.  Handles auto-replacement after
//every space.  Also undoes auto-replacement if the user backspaces over
//the last character entered in the replacement.
class DLLAccess AutoTextEdit : public Edit
{
    public:

        AutoTextEdit( void );

        AutoTextEdit( char const * const pnewLabel,
                      char * const pnewBuffer, 
                      int const newLengthofBuffer,
                      int const newCharacterOffset = 0, 
                      int const Justify = LCD_LEFT_JUSTIFIED );

        //Called by the UI Engine
        RESULT HandleInput( MESSAGE const & message );

    private:

        void TryAutoReplace();

        int  nOriginalEnd;
        int  nOriginalLength;
        int  nReplacementLength;
        int  nSuffixLength;
        char szOriginalText[ 100 ];

};


//Display the AutoText list screen, and allow the user to modify the entries
//manually.
DLLAccess void
AutoTextEntry( void );


#endif //_AUTOTEXT_H_