/****************************************************************************
 BusyStatus.h 
 
 Description:
    This class puts up a status with an hourglass bitmap. It is useful when 
    performing a cpu intensive operation.  Calling BusyStatus::Draw() repeatedly
    during an intensive operation will cause the status box to remain on the 
    screen until the operation terminates.  The box will then be removed from
    the display automatically.
 
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef busystatus_h_included
#define busystatus_h_included

#include "status.h"		// Status definition

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

class DLLAccess BusyStatus : public Status
{
   private:

      int Count;
      int Rate;
      char * String;
      char Buffer[50];
	  bool Enabled;


    public:

        BusyStatus( char * InitString = NULL,
                    int InitTime = 10,
                    int InitCount = 0,
                    int InitRate = 10,
					bool EnableFlag = true)
         : String( NULL )
		 , Enabled( EnableFlag )
        {
            SetBitmap( Bitmaps::HOURGLASS );
            SetText( Buffer );
            Reset( InitString, InitTime, InitCount, InitRate, EnableFlag );
        }

      
        void Reset( char * InitString = NULL,
                    int InitTime = 10,
                    int InitCount = 0,
                    int InitRate = 10,
					bool EnableFlag = true)
        {
            if( InitString ) 
            {
                String = InitString;
            }
            Count = InitCount;
            Rate  = InitRate;
            SetDisplayTime( InitTime );
			Enabled = EnableFlag;
        }
		void Enable( void ) 
        { 
            Enabled = true; 
        }
		void Disable( void )  
        { 
            Enabled = false; 
        }

		void Draw( void )
		{
			
			if( Enabled && ++Count % Rate == 0 ) 
			{
				if( String ) 
				{
					RimSprintf( Buffer, sizeof( Buffer ), "%s\r%4d", String, Count );
				}
				else 
				{
					RimSprintf( Buffer, sizeof( Buffer ), "%4d", Count );
				}
				DisplayModalStatus();
			}
		}



};

#endif // busystatus_h_included
