/*
 * date.h
 */

#ifndef DATE_H
#define DATE_H


#include <memory.h>

#include "pager.h"
#include "type.h"


extern char const * const months[ 13 ];
       
extern byte const num_days_in_month[ 13 ];

inline
byte
get_num_days_in_month( int month, 
                       int year )
{
   return( (byte) ( num_days_in_month[ month ] + ( ( year % 4 ) == 0 ) ) );
   }


inline
const char *
get_month_name( int month )     // 1 to 12: Jan to Dec
{
   return( months[ month ] );
   }
       



class Date {

   private:

      #pragma warning( disable: 4201 )
      
      union {

         struct {
            unsigned short int month : 4;   // 0 to 11
            unsigned short int date  : 5;   // 1 to 31
            unsigned short int year  : 6;   // 0 to 63  (+ 1996) - top bit must be 0!
            };

         unsigned short int data;
         };

      #pragma warning( default: 4201 )


   public:

      Date( const int init_month,
            const int init_date,
            const int init_year )
         : data( 0 )
      {
         month = (unsigned short int) init_month;
         date  = (unsigned short int) init_date;
         year  = (unsigned short int) ( init_year - 1996 );
         }
   
   
      Date( const TIME & time )
         : data( 0 )
      {
         month = (unsigned short int) time.month;
         date  = (unsigned short int) time.date;
         year  = (unsigned short int) ( time.year - 1996 );
         }
   
      
      Date( const int init_data )
         : data( (unsigned short int) init_data )
      {
         }


      Date( void )
         : data( 0 )
      {
         }


      void
      set_date( TIME & time )
      {
         month = (unsigned short int) time.month;
         date = (unsigned short int) time.date;
         year = (unsigned short int) ( time.year - 1996 );
         }


      void
      operator = ( const Date & rhs )
      {
         data = rhs.data;
         }


      void
      operator = ( const int rhs )
      {
         data = (unsigned short) rhs;
         }


      operator unsigned short int ()
      {
         return( data );
         }
         
         
      int
      get_month( void )
      {
         return( month );
         }
         
         
      int
      get_date( void )
      {
         return( date );
         }


      int
      get_year( void )
      {
         return( year + 1996 );
         }


      void
      set_month( int new_month )
      {
         month = (unsigned short int) new_month;
         }


      void
      set_date( int new_date )
      {
         date = (unsigned short int) new_date;
         }


      void
      set_year( int new_year )
      {
         year = (unsigned short int) new_year;
         }

         
      operator == ( const Date & rhs )
      {
         return( data == rhs.data );
         }


      operator != ( const Date & rhs )
      {
         return( data != rhs.data );
         }


      void 
      operator + ( int i )
      {
         date = (unsigned short) ( date + i );
         
         while( date > get_num_days_in_month( month, year ) ) {
            date = 1;
            month++;
            if( month > 12 ) {
               month = 1;
               year++;
               }
            }
         }



      void
      operator ++ ( void )
      {
         operator+( 1 );
         }


      void
      operator - ( int i )
      {
         date = (unsigned short) ( date - i );

         while( date <= 0 ) {
            month--;
            date = get_num_days_in_month( month, year );
            if( month == 0 ) {
               month = 12;
               year--;
               }
            }
         }


      void
      operator -- ( void )
      {
         operator-( 1 );
         }


      int
      operator - ( Date rhs )
      {
         int days = 0;

         if( data < rhs.data ) {
   
            Date temp = rhs;

            while( data != temp.data ) {
               --temp;
               ++days;
               }
            }
         else {

            Date temp = *this;

            while( rhs.data != temp.data ) {
               --temp;
               ++days;
               }
            }

         return( days );
         }

      
      operator TIME ( void )
      {
         TIME time;

         memset( &time, 0, sizeof( time ) );

         time.month = (BYTE) month;
         time.date  = (BYTE) date;
         time.year  = (WORD) ( year + 1996 );

         return( time );
         }
   };
   
   


class Time
{
   private:

      #pragma warning( disable: 4201 )

      union {
        
         struct {
            byte hours;
            byte minutes;
            };
         
         unsigned short data;
         };

      #pragma warning( default: 4201 )


   public:

      Time( byte init_hours,
            byte init_minutes )
         : hours( init_hours )
         , minutes( init_minutes )
      {
         }


      Time( TIME & time )
         : hours( time.hour )
         , minutes( time.minute )
      {
         }


      Time( Time & time )
         : data( time.data )
      {
         }


      Time( void )
         : data( 0 )
      {
         }


      void
      set_time( TIME & time )
      {
         hours = time.hour;
         minutes = time.minute;
         }


      operator unsigned int ( void )
      {
         return( data );
         }


      byte
      get_hours( void )
      {
         return( hours );
         }


      byte
      get_minutes( void )
      {
         return( minutes );
         }


      void
      set_hours( byte new_hours )
      {
         hours = new_hours;
         }


      void
      set_minutes( byte new_minutes )
      {
         minutes = new_minutes;
         }


      operator TIME ( void )
      {
         TIME time;

         memset( &time, 0, sizeof( time ) );

         time.hour   = hours;
         time.minute = minutes;

         return( time );
         }

   };



#endif