/*
 * format_api.h
 */

#ifndef STACK_H
#define STACK_H

#include "pager.h"

#include "type.h"

#include "AddressFieldType.h"

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _MESSAGE_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif


/* 
 * Globals 
 */

extern TASK protocol_stack_task_handle;
const int protocol_stack_stacksize = 3000;


/* 
 * Global Function Prototypes
 */

// Call to start up the transport code
void 
protocol_stack_main( void );


/*
// Function to determine whether the transport layer recognizes 
// Parameters:
//    address_ptr.               points to the address whose possible type(s) should
//                               be returned by function.
//	  address_type				 points to expected communication method type; if 0, then 
//								 is_valid_destination returns tur eif address_ptr matches any 
//								 comm method (use matching_address_types to differentiate
//
//    matching_address_types     points to a bitfield returned by the function which indicates the
//                               possible address types. Bit K is set if the address 
//                               format could correspond to the programmatic, communication 
//                               method with value K defined in addrestype.h. More than one bit 
//                               can be set if the address format matches more than one
//                               communication method. (i.e. to check whether 'PHONE' is a
//                               possible communication method check 
//                               if( *matching_address_types & (1 << PHONE) ) 
//
// Return: 
//    boolean value              true  if address format matches the requested communication                                
//                                     method OR if none specified, matches any comm method
//                               false if address format does not match the requested
//                                     communication method OR if none specified, does not match 
//									   any communication method
// Note: at least one of address_type and matching_address_types must be provided by the 
//		 calling app
//
extern "C" DLLAccess
bool
is_valid_destination( char * address_ptr, 
                      AddressFieldType address_type = ANY_FIELD_TYPE,
					       unsigned long * matching_address_types = NULL);


*/
// Function to return a string describing the communication method
// Paramter:
//    address_type               one of the values defined as a programmatic communication
//                               method in adresstype.h
//
// Return:
//    char *                     a pointer to a NULL terminated string containing a brief
//                               description of the communication method. If the address_type
//                               is not supported a NULL pointer is returned.
//
extern "C" DLLAccess
const char *
get_addresstype_string( AddressFieldType address_field_type );




//
// Transport Error
//

typedef byte TransportError;
   
extern char * transport_error_strings[];
 
inline
char *
get_transport_error_string( TransportError error )
{
   return( transport_error_strings[ error ] );
   }   
 




/*
 * Internal Function Prototypes
 */

//
// Transport Timer support
//

bool
start_radio_retry_timer( int timeout );



bool
start_transport_nack_timer( int timer_id,
                            int timeout );
   
void
stop_transport_nack_timer( int timer_id );


   
bool
start_transport_abort_timer( int timer_id,
                             int timeout );
   
void
stop_transport_abort_timer( int timer_id );

   

#endif