/*
 * queue.h
 */

#ifndef QUEUE_H
#define QUEUE_H

class Queue;

class DLLExport QueueElement
{
   friend Queue;

   protected:
      
      Queue        * queue_ptr;
      QueueElement * next_ptr;
      QueueElement * prev_ptr;


   public:

      QueueElement( void )
         : queue_ptr( NULL )
         , next_ptr( NULL )
         , prev_ptr( NULL )
      {
         }


      void
      remove( void );


      QueueElement *
      get_next_ptr( void )
      {
         return( next_ptr );
         }


      QueueElement *
      get_previous_ptr( void )
      {
         return( prev_ptr );
         }

   };



class DLLExport Queue
{
   protected:

      QueueElement * head_ptr;
      QueueElement * tail_ptr;


   public:

      Queue( void )
         : head_ptr( NULL )
         , tail_ptr( NULL )
      {
         }


      QueueElement *
      get_first_ptr( void )
      {
         return( head_ptr );
         }


      QueueElement *
      get_last_ptr( void )
      {
         return( tail_ptr );
         }


      bool
      is_empty( void )
      {
         return( head_ptr == NULL );
         }


      void
      remove( QueueElement * element_ptr )
      {
         if( element_ptr->prev_ptr != NULL ) {
            element_ptr->prev_ptr->next_ptr = element_ptr->next_ptr;
            }
         else {
            head_ptr = element_ptr->next_ptr;
            }

         if( element_ptr->next_ptr != NULL ) {
            element_ptr->next_ptr->prev_ptr = element_ptr->prev_ptr;
            }
         else {
            tail_ptr = element_ptr->prev_ptr;
            }

         element_ptr->queue_ptr = NULL;
         element_ptr->next_ptr  = NULL;
         element_ptr->prev_ptr  = NULL;
         }


      void
      insert( QueueElement * element_ptr )
      {
         element_ptr->remove();

         if( tail_ptr != NULL ) {
            tail_ptr->next_ptr = element_ptr;
            element_ptr->prev_ptr = tail_ptr;
            tail_ptr = element_ptr;
            }
         else {
            head_ptr = element_ptr;
            tail_ptr = element_ptr;
            }

         element_ptr->queue_ptr = this;
         }


   }; 



inline
void
QueueElement::
remove( void )
{
   if( queue_ptr != NULL ) {
      queue_ptr->remove( this );
      }
   }




#endif