/****************************************************************************
 rimctype.h  -  Properties for characters. Used by both UI Engine and 
                applications
  
Description:
     Explains the purpose of this file. What does this file contain?
 
 Example:
     (Optional) Sample source code of how this module should be used.

 Copyright (C) 1997-1998 Research In Motion Limited
 ***************************************************************************/
#ifndef rimctype_h_included
#define rimctype_h_included
#include "warning.h"
#include <limits.h>



namespace CTYPE
{
	unsigned char const DECIMAL_NUMERIC = 0x01 ;	// #: 0-9
	unsigned char const ALPHABETIC = 0x02;			// A: A-Z
	unsigned char const ALPHANUMERIC = 0x03;		// #/A: Alpha + decimal numeric
	unsigned char const HEXIDECIMAL_NUMERIC = 0x04;	// H:
	unsigned char const PHONE = 0x08;				// P: 0-9, (), -,+,.,x,X
	unsigned char const EMAIL_ADDRESS = 0x10;		// @: all chars except specials, CTLs, SPACE (except @ and .)
													// and comma ',' 
	unsigned char const EMAIL_WORD = 0x20;			// E: all chars except specials, CTLs, SPACE 
	unsigned char const URL = 0x40;					// U: Url characters (a-z,0-9,@,.,#,?,=


    unsigned char const PropertiesforCharacter[256] = {
        // 0
        0x0,    // 0x00
        0x0,    // 0x01
        0x0,    // 0x02
        0x0,    // 0x03
        0x0,    // 0x04
        0x0,    // 0x05
        0x0,    // 0x06
        0x0,    // 0x07
        0x0,    // 0x08
        0x0,    // 0x09
        0x0,    // 0x0a
        0x0,    // 0x0b
        0x0,    // 0x0c
        0x0,    // 0x0d
        0x0,    // 0x0e
        0x0,    // 0x0f
        0x0,    // 0x10
        0x0,    // 0x11
        0x0,    // 0x12
        0x0,    // 0x13
        0x0,    // 0x14
        0x0,    // 0x15
        0x0,    // 0x16
        0x0,    // 0x17
        0x0,    // 0x18
        0x0,    // 0x19
        0x0,    // 0x1a
        0x0,    // 0x1b
        0x0,    // 0x1c
        0x0,    // 0x1d
        0x0,    // 0x1e
        0x0,    // 0x1f
        // 32
        0x48,    // ' ' : (PU)
        0x30,    // '!' : (@E)
        0x0,    // '"'
        0x70,    // '#' : (@EU)
        0x30,    // '$' : (@E)
        0x30,    // '%' : (@E)
        0x30,    // '&' : (@E)
        0x30,    // ''' : (@E)
        0x8,    // '(' : (P)
        0x8,    // ')' : (P)
        0x30,    // '*' : (@E)
        0x38,    // '+' : (P@E)
        0x50,    // ',' : (@U)
        0x38,    // '-' : (P@E)
        0x58,    // '.' : (P@U)
        0x70,    // '/' : (@EU)
        0x7d,    // '0' : (#HP@EU)
        0x7d,    // '1' : (#HP@EU)
        0x7d,    // '2' : (#HP@EU)
        0x7d,    // '3' : (#HP@EU)
        0x7d,    // '4' : (#HP@EU)
        0x7d,    // '5' : (#HP@EU)
        0x7d,    // '6' : (#HP@EU)
        0x7d,    // '7' : (#HP@EU)
        0x7d,    // '8' : (#HP@EU)
        0x7d,    // '9' : (#HP@EU)
        0x40,    // ':' : (U)
        0x0,    // ';'
        0x0,    // '<'
        0x70,    // '=' : (@EU)
        0x0,    // '>'
        0x70,    // '?' : (@EU)
        // 64
        0x50,    // '@' : (@U)
        0x76,    // 'A' : (AH@EU)
        0x76,    // 'B' : (AH@EU)
        0x76,    // 'C' : (AH@EU)
        0x76,    // 'D' : (AH@EU)
        0x76,    // 'E' : (AH@EU)
        0x76,    // 'F' : (AH@EU)
        0x72,    // 'G' : (A@EU)
        0x72,    // 'H' : (A@EU)
        0x72,    // 'I' : (A@EU)
        0x72,    // 'J' : (A@EU)
        0x72,    // 'K' : (A@EU)
        0x72,    // 'L' : (A@EU)
        0x72,    // 'M' : (A@EU)
        0x72,    // 'N' : (A@EU)
        0x72,    // 'O' : (A@EU)
        0x72,    // 'P' : (A@EU)
        0x72,    // 'Q' : (A@EU)
        0x72,    // 'R' : (A@EU)
        0x72,    // 'S' : (A@EU)
        0x72,    // 'T' : (A@EU)
        0x72,    // 'U' : (A@EU)
        0x72,    // 'V' : (A@EU)
        0x72,    // 'W' : (A@EU)
        0x7a,    // 'X' : (AP@EU)
        0x72,    // 'Y' : (A@EU)
        0x72,    // 'Z' : (A@EU)
        0x0,    // '['
        0x40,    // '\' : (U)
        0x0,    // ']'
        0x30,    // '^' : (@E)
        0x30,    // '_' : (@E)
        // 96
        0x30,    // '`' : (@E)
        0x76,    // 'a' : (AH@EU)
        0x76,    // 'b' : (AH@EU)
        0x76,    // 'c' : (AH@EU)
        0x76,    // 'd' : (AH@EU)
        0x76,    // 'e' : (AH@EU)
        0x76,    // 'f' : (AH@EU)
        0x72,    // 'g' : (A@EU)
        0x72,    // 'h' : (A@EU)
        0x72,    // 'i' : (A@EU)
        0x72,    // 'j' : (A@EU)
        0x72,    // 'k' : (A@EU)
        0x72,    // 'l' : (A@EU)
        0x72,    // 'm' : (A@EU)
        0x72,    // 'n' : (A@EU)
        0x72,    // 'o' : (A@EU)
        0x72,    // 'p' : (A@EU)
        0x72,    // 'q' : (A@EU)
        0x72,    // 'r' : (A@EU)
        0x72,    // 's' : (A@EU)
        0x72,    // 't' : (A@EU)
        0x72,    // 'u' : (A@EU)
        0x72,    // 'v' : (A@EU)
        0x72,    // 'w' : (A@EU)
        0x7a,    // 'x' : (AP@EU)
        0x72,    // 'y' : (A@EU)
        0x72,    // 'z' : (A@EU)
        0x30,    // '{' : (@E)
        0x30,    // '|' : (@E)
        0x30,    // '}' : (@E)
        0x30,    // '~' : (@E)
        0x0,    // 0x7f
        // 128
        0x0,    // 0x80
        0x0,    // 0x81
        0x0,    // 0x82
        0x0,    // 0x83
        0x0,    // 0x84
        0x0,    // 0x85
        0x0,    // 0x86
        0x0,    // 0x87
        0x0,    // 0x88
        0x0,    // 0x89
        0x0,    // 0x8a
        0x0,    // 0x8b
        0x0,    // 0x8c
        0x0,    // 0x8d
        0x0,    // 0x8e
        0x0,    // 0x8f
        0x0,    // 0x90
        0x0,    // 0x91
        0x0,    // 0x92
        0x0,    // 0x93
        0x0,    // 0x94
        0x0,    // 0x95
        0x0,    // 0x96
        0x0,    // 0x97
        0x0,    // 0x98
        0x0,    // 0x99
        0x0,    // 0x9a
        0x0,    // 0x9b
        0x0,    // 0x9c
        0x0,    // 0x9d
        0x0,    // 0x9e
        0x0,    // 0x9f
        // 160
        0x0,    // 0xa0
        0x0,    // 0xa1
        0x0,    // 0xa2
        0x0,    // 0xa3
        0x0,    // 0xa4
        0x0,    // 0xa5
        0x0,    // 0xa6
        0x0,    // 0xa7
        0x0,    // 0xa8
        0x0,    // 0xa9
        0x0,    // 0xaa
        0x0,    // 0xab
        0x0,    // 0xac
        0x0,    // 0xad
        0x0,    // 0xae
        0x0,    // 0xaf
        0x0,    // 0xb0
        0x0,    // 0xb1
        0x0,    // 0xb2
        0x0,    // 0xb3
        0x0,    // 0xb4
        0x0,    // 0xb5
        0x0,    // 0xb6
        0x0,    // 0xb7
        0x0,    // 0xb8
        0x0,    // 0xb9
        0x0,    // 0xba
        0x0,    // 0xbb
        0x0,    // 0xbc
        0x0,    // 0xbd
        0x0,    // 0xbe
        0x0,    // 0xbf
        // 192
        0x0,    // 0xc0
        0x0,    // 0xc1
        0x0,    // 0xc2
        0x0,    // 0xc3
        0x0,    // 0xc4
        0x0,    // 0xc5
        0x0,    // 0xc6
        0x0,    // 0xc7
        0x0,    // 0xc8
        0x0,    // 0xc9
        0x0,    // 0xca
        0x0,    // 0xcb
        0x0,    // 0xcc
        0x0,    // 0xcd
        0x0,    // 0xce
        0x0,    // 0xcf
        0x0,    // 0xd0
        0x0,    // 0xd1
        0x0,    // 0xd2
        0x0,    // 0xd3
        0x0,    // 0xd4
        0x0,    // 0xd5
        0x0,    // 0xd6
        0x0,    // 0xd7
        0x0,    // 0xd8
        0x0,    // 0xd9
        0x0,    // 0xda
        0x0,    // 0xdb
        0x0,    // 0xdc
        0x0,    // 0xdd
        0x0,    // 0xde
        0x0,    // 0xdf
        // 224
        0x0,    // 0xe0
        0x0,    // 0xe1
        0x0,    // 0xe2
        0x0,    // 0xe3
        0x0,    // 0xe4
        0x0,    // 0xe5
        0x0,    // 0xe6
        0x0,    // 0xe7
        0x0,    // 0xe8
        0x0,    // 0xe9
        0x0,    // 0xea
        0x0,    // 0xeb
        0x0,    // 0xec
        0x0,    // 0xed
        0x0,    // 0xee
        0x0,    // 0xef
        0x0,    // 0xf0
        0x0,    // 0xf1
        0x0,    // 0xf2
        0x0,    // 0xf3
        0x0,    // 0xf4
        0x0,    // 0xf5
        0x0,    // 0xf6
        0x0,    // 0xf7
        0x0,    // 0xf8
        0x0,    // 0xf9
        0x0,    // 0xfa
        0x0,    // 0xfb
        0x0,    // 0xfc
        0x0,    // 0xfd
        0x0,    // 0xfe
        0x0     // 0xff
    };

    bool IsA(char const CharactertoCheck, unsigned char const Props);

 
} //end CTYPE namespace


#endif	// rimctype_h_included
