/****************************************************************************
 utilities.h  -  Utility routines used by both UI Engine and applications
                 Some of the common string run-time routines are re-implemented
                 order to avoid bringing in libraries that aren't in the embedded
                 system
 

 Copyright (C) 1997-1998 Research In Motion Limited
 ***************************************************************************/

#ifndef utilities_h_included
#define utilities_h_included
#include "warning.h"
#include "basetype.h"

//type safe min and max functions for use with classes that define the >, < and ?: operators

template< class T >
inline
T &
MIN( T const & a, T const & b )
{
	return( a < b ? a : b );
}


template< class T >
inline
T &
MAX( T const & a, T const & b ) 
{
    return( a > b ? a : b );
}                           


// small integer (0 to 999) to string conversion
char * sitoa( int value, char * string ); 


inline char tolower( char const ch )
{
    return( (char) ( (unsigned) ch - 'A' >= 26 ? ch : ch - 'A' + 'a' ) );
}


inline char toupper( char const ch )
{
    return( (char) ( (unsigned) ch - 'a' >= 26 ? ch : ch - 'a' + 'A' ) );
}


char * strncpy( char * dest, int dest_length, const char * src, int src_length );
char * strcpy( char * dest, int dest_length, const char * src );
char * strcat( char * dest, int dest_length, const char * src );
int strncmp( const char * str1, const char * str2, int len );
int strnicmp( const char * str1, const char * str2,int len );
int rim_stricmp( const char * str1, const char * str2 );
bool prefix_match( char const * string, char const * prefix );
bool prefix_match_i( char const * string, char const * prefix );
char * RimStristr( const char * text, const char * pattern, int text_length=-1 );

#define RimStristrTerm(a,b,c)	RimStristr((a),(b),((c)-(a)))

char * RimStrstr( const char * text, const char * pattern );
long RimStrtol( const char *nptr, const char **endptr, int ibase );


inline int atoi( char *buffer, int radix = 10 )
{
    return( RimStrtol( buffer, NULL, radix ) );
}

#endif