/****************************************************************************
 Link.h
 
 Description:
    This file defines the backup/restore interface to the serial port that 
    the Options application provides to the other applications on the device.
    This interface is used by the address book and AutoText.
 
 Example:
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef LINK_H
#define LINK_H

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _OPTIONS_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif

typedef unsigned short    REPLY;


// These values are currently under review
#define REPLY_OK					0x0000

// link layer commands/errors:
enum {

	LINK_ERROR = 0x8000,
	LINK_CONNECTED, 
	LINK_DISCONNECTED,
	LINK_TIMEOUT_ERROR,
	LINK_COMM_ERROR,
	ID_UNKNOWN,		
	INVALID_PASSWORD,

};

// application layer errors:
enum {

	APP_ERROR = 0xa000,
	APP_INVALID_DATA,   		// invalid packet data
	APP_UNKNOWN_COMMAND,		// command unknown
	APP_UNKNOWN_RECORD, 		// record not found in DB
	APP_OUT_OF_MEMORY,  		// no more memory available
	APP_SHORT_PACKET,   	    // not enough data from host (length mismatch)

};


// macros to determine error location:
#define	ERROR_LAYER_MASK			0xf000					// top 4 bits identify source of error
#define is_app_error(err)			(((err)&ERROR_LAYER_MASK)==APP_ERROR)
#define is_link_error(err)			(((err)&ERROR_LAYER_MASK)==LINK_ERROR)

#define	ERROR_CODE_MASK				0x0fff						// bottom 12 bits identify error code
#define error_code_index(err)		(((err)-1)&ERROR_CODE_MASK)		// offset: 1->n ==>  0->n-1

typedef int SC_HANDLE;
extern "C" DLLAccess SC_HANDLE RegisterBackupRestore( char *name, REPLY (*func)(int from, BYTE *buffer, int length, bool more) );
extern "C" DLLAccess REPLY SendData( SC_HANDLE ID, BYTE *data, int length, bool flush );
extern "C" DLLAccess WORD Crc16( WORD crc, BYTE *data, int length );

#endif
