/****************************************************************************
 Options.h
 
 Description:
    Defines the API to the Options application.  Allows any application to 
    display the list of options that can be modified, and to register with the
    options subsystem to have a string added to that list.  When the user selects
    that string, a function will be called and the user may display a screen of
    options.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef OPTIONS_H
#define OPTIONS_H

#ifdef DLLAccess
#undef DLLAccess
#endif
#ifdef _OPTIONS_
#define DLLAccess DLLExport
#else 
#define DLLAccess DLLImport
#endif

#include "choice.h"

#define FORMAT_SHORT      0
#define FORMAT_LONG       1  

// Display Of App Name/Version
extern "C" DLLAccess int  RegisterApplication( char *name, int major, int minor, int build );
extern "C" DLLAccess void RegisterStartupTask( void );

#ifdef _USE_RIBBON_
extern "C" DLLAccess void DisplayOptions( void );
#endif

// Register for Options
enum {

	OPTION_DEVICE,
	OPTION_NOTIFY,
};

extern "C" DLLAccess int RegisterOptions( int section, Choice *field );


// Returns a string of the Option's DLL title or NULL if failed to register
extern "C" DLLAccess char *OptionsEntry( void (*func)(void), char *name );

// Date & Time require a 20 byte buffer length, 'form' is long or short form 
// If time is NULL, returns the current system time/date. 
extern "C" DLLAccess void FormatDate( TIME *date, char *buffer, int form );
extern "C" DLLAccess void FormatTime( TIME *time, char *buffer, int form );
extern "C" DLLAccess void FormatDay( TIME *time, char *buffer, int form );
extern "C" DLLAccess bool OptionsIsDaylightSavingsEnabled( void );
extern "C" DLLAccess int  OptionsGetCurrentTimeZone( void );

// Screen Saver Message
// Returns 1 if successful, otherwise 0 failed to display message
extern "C" DLLAccess int ScreenSaverMsg( TASK task, char *msg, int priority, void (*func)(void) );
extern "C" DLLAccess void DisplayScreenSaver( void );

// Security - returns 1 if registered, otherwise 0 failed to register.
extern "C" DLLAccess int RegisterSecurity( void (*func)(void), char * message );

#endif

