/*-------------------------------------------------------------------------*
 *
 * Rim.h
 *  This file defines the RIM API.
 *
 * Copyright 1997-1998 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/

#ifndef __RIM_H
#define __RIM_H

#pragma pack(push,2)

#ifdef __cplusplus
extern "C" {
#endif

#include "BaseType.h"


/*-------------------------------------------------------------------------*
 * Constants.
 *-------------------------------------------------------------------------*/

/* Maximum number of applications supported. */
#define MAX_APPLICATIONS 31

/* Devices. */
#define DEVICE_SYSTEM           1       // THE OPERATING SYSTEM
#define DEVICE_KEYPAD           2       // KEYPAD
#define DEVICE_RTC              3       // REAL TIME CLOCK
#define DEVICE_TIMER            4       // OS TIMER
#define DEVICE_COM1             5       // EXTERNAL SERIAL PORT
#define DEVICE_RADIO            6       // RADIO DEVICE
#define DEVICE_HOLSTER          7       // HOLSTER

#define LAST_DEVICE             DEVICE_HOLSTER

/* SYSTEM events. */
#define SWITCH_FOREGROUND       0x0100  // SWITCH TO FOREGROUND PROCESSING
#define SWITCH_BACKGROUND       0x0101  // SWITCH TO BACKGROUND PROCESSING
#define BATTERY_LOW             0x0102  // BATTERY LOW
#define POWER_OFF               0x0103  // PAGER IS ABOUT TO "POWER OFF"
#define POWER_UP                0x0104  // COMING OUT OF POWER_OFF STATE
#define TASK_LIST_CHANGED       0x0106  // TASK CREATED, DESTROYED or SETPID
#define BATTERY_GOOD            0x0107  // BATTERY BACK TO NORMAL
#define MEMORY_LOW              0x0108  // LOW MEMORY, PLEASE FREE UP RAM

/* RTC events. */
#define RTC_ALARM_EXPIRED       0x0300
#define RTC_CLOCK_UPDATE        0x0301

/* HOLSTER events. */
#define OUT_OF_HOLSTER          0x0700
#define IN_HOLSTER              0x0701

/* Timer modes. */
#define TIMER_PERIODIC 1
#define TIMER_ONE_SHOT 2
#define TIMER_ABSOLUTE 3

/* Timer tick rate. */
#define TICKS_PER_SECOND       100

/* Requested task was not found. */
#define TASK_NOT_FOUND         ((TASK)0xffffffff)

/* Notification modes. */
//#define NO_NOTIFY               0x00
#define TONE_ALERT              0x01
#define VIBRATE_ALERT           0x02
#define VIBRATE_AND_TONE        0x03
#define VIBRATE_THEN_TONE       0x04

/* Vibrate modes. */
#define PULSE_VIBRATE           0x00
#define CONTINUOUS_VIBRATE      0x01

/* Tone modes. */
#define KILL_NOTIFY             -1
#define NO_NOTIFY               0
#define TONE_SEQUENCE_1         1
#define TONE_SEQUENCE_2         2
#define TONE_SEQUENCE_3         3
#define TONE_SEQUENCE_4         4
#define TONE_SEQUENCE_5         5
#define TONE_SEQUENCE_6         6
/* Message Bit Field Defs */
#define MC_DEVICE    0x01
#define MC_EVENT     0x02
#define MC_SUBMSG    0x04
#define MC_MASK_TAIL 0x08


/*-------------------------------------------------------------------------*
 * Types.
 *-------------------------------------------------------------------------*/

/* Message Type */
typedef struct {
    DWORD   Device;         // WHO CAUSED THE EVENT
    DWORD   Event;          // WHAT TYPE OF EVENT TOOK PLACE
    DWORD   SubMsg;         // ANY SUB MESSAGE (OPTIONAL)

    DWORD   Length;         // LENGTH OF DATA  (OPTIONAL)
    char   *DataPtr;        // POINTER TO DATA (OPTIONAL)

    DWORD   Data[2];        // DATA STORAGE (OPTIONAL)
} MESSAGE;

/* Time Type */
typedef struct              // USED FOR GET/SET DATE and TIME
{
    BYTE    second;
    BYTE    minute;
    BYTE    hour;
    BYTE    date;
    BYTE    month;
    BYTE    day;            // day of week, Sun=0, Mon=1, ..., Sat=6
    WORD    year;
    BYTE    TimeZone;
} TIME;

/* Process Information Descriptor (PID) */
typedef struct {
    const char   *Name;
    BOOL          EnableForeground;
    const BitMap *Icon;
} PID;

/* Notification Type */
typedef struct {
    BYTE    duty;
    BYTE    volume;
    BYTE    maxVolume;
    BYTE    repetitions;
    BYTE    vibrateTime;
    BYTE    vibrateType;
    BYTE    inHolsterNotify;
    BYTE    outOfHolsterNotify;
    BYTE    pause;
} AlertConfiguration;


/*-------------------------------------------------------------------------*
 * Function Prototypes.
 *-------------------------------------------------------------------------*/

typedef int (_cdecl * CALLBACK_FUNC)(MESSAGE *);

void    __cdecl RimAlertNotify(int Notify, int MaxRepetitions);
TASK    __cdecl RimCreateThread(void (*Entry)(void), DWORD Stacksize);
BOOL    __cdecl RimDisableAppSwitch(void);
BOOL    __cdecl RimEnableAppSwitch(void);
TASK    __cdecl RimFindTask(const char * Prefix);
void    __cdecl RimFree(void * Block);
TASK    __cdecl RimGetAlarm(TIME * Time);
void    __cdecl RimGetAlertConfiguration(AlertConfiguration * AlertConfig);
DWORD   __cdecl RimGetBatteryLevel(void);
TASK    __cdecl RimGetCurrentTaskID(void);
void    __cdecl RimGetDateTime(TIME * Time);
TASK    __cdecl RimGetForegroundApp(void);
long    __cdecl RimGetIdleTime(void);
int     __cdecl RimGetLanguage(void);
TASK    __cdecl RimGetMessage(MESSAGE * Msg);

TASK    __cdecl RimGetMessageRaw(MESSAGE * Msg);
BOOL    __cdecl RimRegisterMessageCallback(DWORD MessageBits, DWORD MaskBits, CALLBACK_FUNC HandlerFunc);
DWORD   __cdecl RimGetOSversion(void);
BOOL    __cdecl RimGetPID(TASK hTask, PID * Pid, const char ** Subtitle);
long    __cdecl RimGetTicks(void);
int     __cdecl RimHolsterStatus(void);
void    __cdecl RimInvokeTaskSwitcher(void);
void    __cdecl RimKillTimer(DWORD TimerID);
DWORD   __cdecl RimMemoryRemaining(void);
void  * __cdecl RimMalloc(DWORD Size);
int     __cdecl RimPagerSetup(int CallType, void * Pointer);
BOOL    __cdecl RimPeekMessage(void);
void    __cdecl RimPostMessage(TASK HTask, const MESSAGE * Msg);

void  * __cdecl RimRealloc(void * Ptr, DWORD Size);
BOOL    __cdecl RimRequestForeground(TASK HTask);
BOOL    __cdecl RimRequestPowerOff(void);
void    __cdecl RimSetReceiveFromDevice(const DWORD Device, BOOL ReceiveFrom );
BOOL    __cdecl RimSleep(DWORD Ticks);
void    __cdecl RimToggleMessageReceiving(BOOL ReceiveMessages);
BOOL    __cdecl RimSendSyncMessage(TASK HTask, const MESSAGE * Msg, 
                                               const MESSAGE * ReplyMsg );
BOOL    __cdecl RimReplyMessage(TASK HTask, const MESSAGE * Msg );
TASK    __cdecl RimWaitForSpecificMessage(MESSAGE * Msg, MESSAGE * Compare, 
                                          DWORD Mask );
BOOL    __cdecl RimSendMessage(TASK HTask, const MESSAGE * Msg);

BOOL    __cdecl RimSetAlarmClock(TIME * Time, BOOL Enable);
void    __cdecl RimSetAlertConfiguration(const AlertConfiguration * AlertConfig);
BOOL    __cdecl RimSetDate(const TIME * Time);
void    __cdecl RimSetPID(const PID * Pid);
void    __cdecl RimSetLanguage(int Language);
const char * __cdecl RimSetSubTitle(const char * NewTitle);

BOOL    __cdecl RimSetTime(const TIME * Time);
BOOL    __cdecl RimSetTimer(DWORD TimerID, DWORD Time, DWORD Type);

void    __cdecl RimSpeakerBeep(int Frequency, int Duration, int Duty, int Volume);
DWORD   __cdecl RimStackUsage(void);
void    __cdecl RimTaskYield(void);
void    __cdecl RimTerminateThread(void);
void    __cdecl RimTestAlert(int notify, AlertConfiguration * TrialConfig);

void    __cdecl RimCatastrophicFailure(const char * FailureMessage);
void    __cdecl RimInitiateReset(void);

int     __cdecl RimSprintf (char * Str, int Maxlen, const char * Fmt, ...);

#ifndef _VA_LIST_DEFINED
typedef char * va_list;
#define _VA_LIST_DEFINED
#endif
int     __cdecl RimVsprintf(char * Buf, int Maxsize, const char * Fmt, va_list Argp);

void    __cdecl RimDebugPrintf(const char * String, ...);
BOOL    __cdecl RimPassword(void * PasswordData, int Length);


#ifdef __cplusplus
}
#endif

#pragma pack(pop)

#endif  /* __RIM_H */
