/***************************************************************************
 YesNoDialog.h

 Description:
    A YesNoDialog is a rectangular dialog box that contains a text string and
    a list containing the strings "Yes", "No", and optionally "Cacnel".  The 
    dialog will not be dismissed until the user highlights one of the options 
    and clicks the thumb wheel.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "UI32.h"
#include "warning.h"

#pragma data_seg( ".text" )

char const * const YesNoChoices[ 3 ] = { "Yes", "No", NULL };
char const * const YesNoCancelChoices[ 4 ] = { "Yes", "Cancel", "No", NULL };

#pragma data_seg()

class DLLAccess YesNoDialog: public Dialog
{
    public:

        typedef enum {
            ITEM_YES = 0,
            ITEM_NO = 1,
            ITEM_CANCEL = 2
        } YN_ITEM;

        YesNoDialog( char const * const pQuestion   = NULL,
                     YN_ITEM            DefaultItem = ITEM_YES,
                     bool               fShowCancel = false );

        //Displays the dialog and returns ITEM_YES, ITEM_NO, or ITEM_CANCEL
        virtual YN_ITEM Go(UIEngine &UIEngineToUse );

        //Handles the input passed in by the UI Engine
		RESULT  HandleInput(MESSAGE const &InputMessage);

        //Set the text string to be used for the question
        bool SetQuestion( char const * const pQuestion )
            {
                m_pQuestion = pQuestion;
                return true;
            }

        //Set the item to be selected by default
        bool SetDefaultItem( YN_ITEM DefaultItem )
            {
                m_DefaultItem = DefaultItem;
                return true;
            }

        //If fShowCancelItem is true, CANCEL is displayed in the list.  Otherwise,
        //Cancel is not displayed.
        bool ShowCancelItem( bool fShowCancelItem = true )
            {
                m_fShowCancelItem = fShowCancelItem;
                
                int nNumLines = fShowCancelItem ? 3 : 2;
                
                m_StringList.SetStringList( ( fShowCancelItem ? YesNoCancelChoices : YesNoChoices ) );
                SetField( m_StringList, nNumLines * Services::GetFontHeight() );
    
                return true;
            }
		
        //Returns the item that was selected last
        YN_ITEM GetAnswer()
			{ 
				return m_DefaultItem;
			}
		
        //Returns true if the item that was selected last was NO
        bool AnswerIsYes()
			{
				return (m_DefaultItem == ITEM_YES)? TRUE : FALSE;
			}

        //Returns true if the item that was selected last was YES
		bool AnswerIsNo()
			{
				return (m_DefaultItem == ITEM_NO)? TRUE : FALSE;
			}


    protected:

        char const*        m_pQuestion;
        YN_ITEM            m_DefaultItem;
        bool               m_fShowCancelItem;

        StringList         m_StringList;

};






