/****************************************************************************
 Area.h  
 
 Description:
    Defines a rectangular area on the display, and indirects operations to
    that portion of the display.  All points specified in the function calls
    are offsets from the upper left corner of the area.
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef area_h_included
#define area_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"

class Area;

#include "pager.h"
#include "helper.h"

class DLLAccess Area
{   
  public:                   
    
    //Constructors that allow areas to be created in different ways.
    Area(); 

    Area( XYRect const & RectToUse );

    Area( XYPoint const & TopLeft, XYPoint const & BottomRight );

    //Places a bitmap or string within the Area.  Start is a point with respect to the 
    //upper left corner of the Area, and object's upper left corner will be placed 
    //at that point.
    void PutBitmapAt( XYPoint const & Start, BitMap const * const pBitmap ) const;
    void PutBitmapAt( XYPoint const & Start, int const Width, int const Height, BitMap const * const pBitmap ) const;
    void PutStringAt( XYPoint const & Start, char const * const pBuffer, int const Length) const;

    //Draws a line between the points indicated.  The points are with respect to
    //the upper left corner of the Area.
    void DrawLine( XYPoint const & Start, XYPoint const & End, int DrawingMode=DRAW_BLACK ) const;

    //Clears and outlines a rectangle within the Area.  If an XYRect is specified, 
    //that rectangle is taken with respect to the upper left corner of the Area.
    //If no XYRect is specified, the entire Area is cleared and outlined.
    void DrawRect( XYRect const & RectToDraw )const;
    void DrawRect()const;

    //Clears a rectangle within the Area.  If an XYRect is specified, 
    //that rectangle is taken with respect to the upper left corner of the Area.
    //If no XYRect is specified, the entire Area is cleared.
    void ClearRect( XYRect const & RectToClear )const;
    void ClearRect()const;

    //Inverts a rectangle within the Area.  If an XYRect is specified, 
    //that rectangle is taken with respect to the upper left corner of the Area.
    //If no XYRect is specified, the entire Area is inverted.
    void InvertRect( XYRect const & RectToInvert )const;
    void InvertRect()const;

    //Sets the dimensions of the area.  The old area will not be erased if the new one
    //does not cover it completely.
    void SetDimensionsofArea( XYRect const & NewSize );
    
    //Returns the rectangle the area represents.
    XYRect GetRect() const;
	
    //Returns the height and width of the rectangle, in pixels.
    int Height() const; 
	int Width() const;
  
  protected:
    XYRect RectOfArea; 

};

#endif //area_h_included
