/****************************************************************************
 Buffer.h

 Description:
    Buffer is a worker for the edit class. The buffer class is used internally
    to abstract a string buffer. It allows for strings to be created from stack
    char *'s or grow and shrink (up to a maximum size).
      
 
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef buffer_h_included
#define buffer_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
#include "pager.h"


class DLLAccess Buffer
{
    public: // types
        typedef enum
        {
            SUCCESS,
            NOT_INITIALIZED,
            OUT_OF_MEMORY,
            MAX_SIZE_REACHED,
            READ_ONLY,
        }RESULT;
        enum
        {
            NUMBER_OF_BOOKMARKS=4
        };

    public: // initialization functions used by edit object

        // Constructor    
        Buffer();

        // Destructor    
        ~Buffer();

        // read/write application allocated buffer
        void SetBuffer( char * const newpFixedBuffer, int Size );
        
        // read only application allocated buffer 
        void SetBuffer( char const * const newpFixedBufferReadOnly, int Size );

        //read/write UI Engine allocated buffer
        RESULT SetBuffer( int const newInitialSize, int const newMaxSize );

        // used by edit to determining "special" offsets in the buffer (e.g. last place 
        // where auto capitialization occurred
        void SetBookmark( int const Offset = -1, int BookmarkNumber = NUMBER_OF_BOOKMARKS-1 );
        int GetBookmark( int BookmarkNumber = NUMBER_OF_BOOKMARKS -1 ) const;

    
    public: // manipulation functions

        // inserting character/strings in the buffer
        RESULT Insert( char const Character, int const AtPosition );
        RESULT Insert(char const * const pInsertString, int const Length, int const AtPosition );
        void Delete( int const Offset, int const Length = 1 );
        int GetLength() const;
        operator char const * ()const;
        char const * GetBuffer( bool SubstituteSymbols = false ) const;

        bool Grow( int const MinimumRequired );
        bool Shrink();

        void ClearReadOnly();

    protected:

        int BookmarkArray[NUMBER_OF_BOOKMARKS];
        int CurrentSize; // current size reflects the current length of string + null
        int CurrentMaxSize; // current max size reflects the largest number current size can be 
                            // without growing the buffer (as allowed)
        int MaxSize; // max size is 0 if fixed buffer, non-zero for dynamic

        char *pBuffer; // pBuffer is set if there is a buffer defined, however, it is 
                       // superseded if pReadOnlyBuffer is set
        char const * pBufferReadOnly; // when making a READONLY buffer, pBufferReadOnly is set,
                                      // but pBuffer is not cleared
        


};

      
#endif // buffer_h_included
