/****************************************************************************
 Dialog.h
 
 Description:
    Dialog boxes are rectangular containers that are application modal and 
    appear in the middle of the LCD display. A dialog box has three components:
        1. An optional Bitmap
        2. A Text field, and
        3. either a List field, Edit field, or Choice field.
    When the dialog box appears, the List/Edit/Choice field is highlighted. 
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef dialog_h_included
#define dialog_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
class Dialog;
#include "result.h"
#include "uiengine.h"
#include "pager.h"
#include "manager.h"
#include "predefinedbitmaps.h"

class DLLAccess Dialog : public FieldManager
{
    public: // used by client
        Dialog();
        Dialog(char const * const newpDisplayString,BitMap const * const newpBitmap = NULL, Field * const pField = NULL);
        Dialog(char const * const newpDisplayString,Bitmaps::PREDEFINED_BITMAP const PredefinedBitmap,Field * const pField = NULL);


		virtual ~Dialog();
    
        virtual void DisplayDialog();
	    void ClearDialog();
	    void SetField( Field &newpField , int const newFieldHeight = LCD_HEIGHT, int const newFieldWidth = LCD_WIDTH);
        void SetDisplayString(char const * const newpDisplayString );
        void SetBitmap(BitMap const * const newpBitmap);
        void SetBitmap(Bitmaps::PREDEFINED_BITMAP const PredefinedBitmap);
        void SetAnchorPoint( int AnchorPoint );
		
	public: // used by UI Engine
        void ResetDimensionDependentData();
        int GetLCDParametersUsed();
        void SetLCDParametersUsed();
        MANAGERTYPE GetManagerType();

		void SetUIEngine(UIEngine & newCallingUI);
        void ClearUIEngine();
		virtual RESULT HandleInput(MESSAGE const &InputMessage);

        enum
		{
				MARGIN = 6, // space around outside
				BORDER = 1, // space around line
				SPACER = 2  // space between items
		};

    protected:  
        UIEngine * pCallingUIEngine;
        int LastLCDParametersUsed;
        void CalculateLayout();
        bool IsFieldManagerDisabled() {return false;}
        bool FieldManagerDisabled();

        BitMap const * pBitmap;
        char const * pDisplayString;

        XYRect TotalRectangle;
        
        XYPoint BitmapTopLeft;
        XYPoint TextTopLeft;
        XYRect FieldRect;
        
        Field * pField;

        int FieldandTextWidth;
        int FieldHeight; //greater than font height, defaults to largest possible
        int AnchorPoint;
    
        
};


      
#endif // dialog_h_included