/****************************************************************************
 Helper.h
 
 Description:
    This file contains various classes that simplify operations within the 
    UI Engine.
    The XYPoint class is used to store x,y coord.
    The Rect class is used to store the top left and bottom right points of 
    a rectangle.
 
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef helper_h_included
#define helper_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"

class Rect;
class XYPoint;

#include "Pager.h"

class DLLAccess XYPoint
{    
	public:

			
		XYPoint( int const NewX, int const NewY );
		XYPoint();
		
		XYPoint& operator =( const XYPoint& p );
		friend XYPoint operator +( const XYPoint& p1, const XYPoint& p2 ); //for vector style addition
        friend XYPoint operator -( const XYPoint& p1, const XYPoint& p2 ); //for vector style addition

		int& X(); //can be used to change location of point
		int& Y(); //can be used to change location of point
		             

    
	protected:
		int x;
		int y;
};

class DLLAccess XYRect
{

  public:
	
    XYRect()
	:TopLeft()
	,BottomRight()
	{}
    
    XYRect( XYPoint NewTopLeft, XYPoint NewBottomRight )
    {
        TopLeft = NewTopLeft;
        BottomRight = NewBottomRight;
    }

	int Height() const;
	int Width() const;

	int X1() const;
	int Y1() const;

    int X2() const;
    int Y2() const;

	XYPoint GetTopLeft() const ;
	void SetTopLeft( XYPoint newTopLeft );

    XYPoint GetBottomRight() const;
	void SetBottomRight( XYPoint newBottomRight );
	
	friend XYRect operator +( const XYRect& r,const XYPoint& vect );
    friend XYRect operator -( const XYRect& r,const XYPoint& vect );
    XYRect & operator =(XYRect newRect);

  protected:
    XYPoint TopLeft;
    XYPoint BottomRight;
    
};
      


#endif // helper_h_included