/****************************************************************************
 Label.h
 
 Description:
    The label class is a box that contains static text.  It is the parent class
    of the title class, which defines a text string that is displayed on the top
    line of the display. 
 
 Example:
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef label_h_included
#define label_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"

class Label;


#include "helper.h"
#include "area.h"
#include "linkedlist.h"

class DLLAccess Label : public LinkedListEntry
{
	public: //used by UIEngine
        void Redraw();
        void SetText( char const * const pnewText );
        void SetLocation( XYRect const &rect );
        void SetJustification(int const newJustification = LCD_LEFT_JUSTIFIED );
        void SetTruncation(bool const newTruncateRight = true);
        char const * GetText();
        void SetLabelDisabled();
        void SetLabelEnabled();



    public: //used internally
        Label();
        void DisplayLabel();
        void UnDraw();
        void ResetDimensionDependentData();
    protected:
		char const* pText;
		Area LabelArea;
        bool OnLCDFlag;
        int Justification;
        bool TruncateRight;
        bool LabelDisabled;

};


class DLLAccess Title : public Label
{
    public: //used by client
        Title (int const XPosition = 0,
               int const Width = LCD_WIDTH,
               char const * const pnewText = NULL,
               int const Justify = LCD_LEFT_JUSTIFIED);
        
        void SetLocation ( int const XPosition, int const Width);


};

     
#endif