/****************************************************************************
 List.h
 
 Description:
    Lists are fields that present related yet separate items on each line. 
    When a line in the field is selected, the cursor takes up the entire line.
	The list can be changed programmatically causing changes to smoothly be 
	incorporated into the user's navigation.  When the UI Engine needs to know
    what to display, it will call the pure virtual member function NowDisplaying(),
    passing in the two-byte handle corresponding to the list entry being shown.
    From within this function, the application calls the member function PutColumn 
    to actually display the data.
	      
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/
 
#ifndef list_h_included
#define list_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"


class List;
class StringList;
typedef BOOL boolean;

typedef short ListHandle;

typedef enum
{
	DEFAULT
} COLUMN_FLAGS;

#include <limits.h>
#include "field.h"
#include "uiengine.h"

      
class DLLAccess List : public Field
{

	public: // used by client
		List();
		List( int const NumEntries );
        virtual void Initialize();
		virtual void Redraw();
		virtual void Redraw( int const Index );
		
        int GetSelectedIndex( int * pFirst = NULL) const;
        void SetSelectedIndex(int const Index); 
        void SetNumEntries( int const NumEntries );
		int GetNumEntries() const; 
        void SetTopLineEntry( int const Index );
		int GetTopLineEntry() const;
        void EnableMultipleSelection();
		
        void SetJustification( int const Justify=LCD_LEFT_JUSTIFIED );  
		
        void Insert( int const Index, bool Redraw = true );
		void Delete( int const Index, bool Redraw = true );
        
        
        virtual void NowDisplaying( int const Index ) = 0;
		void PutColumn( char const * const pnewStr, int const ColumnWidth = LCD_WIDTH, 
						int const Flags = DEFAULT, int const StrLength = INT_MAX );
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );

	public: // used by UI Engine
		//ancestor functions
		void ResetDimensionDependentData(); //used for updating display parms on the fly    
		int QueryPixelHeight(FieldLocation const StartFrom, int const MaximumRelevant);
        int QueryHeightInArea( FieldLocation const StartFrom );
        int QueryLineHeight(int const MaximumRelevant);
		bool IsFocusVisible(); 
        
        void Draw( int const PixelOnScreen, FieldLocation const AnchoredFrom, int &TopPixel, 
                   int &HeightinPixels, 
                   bool &MoreDataAtTop, bool &MoreDataAtBottom );
        void UnDraw();
        bool GiveFocus(int const DirectionMagnitude);
		void TakeFocus();

		RESULT HandleInput(MESSAGE const &InputMessage);

	protected:

		
		RESULT HandleScroll(MESSAGE const &InputMessage);
		int GetNumEntries( int const OffsetWithSelection ) const;


		int OffsetWithSelection;
		int RedrawItemOffset;
        bool FirstItemRedraw;

		bool OnLCDFlag;
		bool FocusFlag;
		char * PutColumnString;
		int NumEntries;
		int StartingIndexinListToDraw;
		int DirectionMagnitude;
		int InvertLineOnDisplay;
		XYPoint PointforString;
		int Justify;
        int SavedOffset;
        int FirstinMultipleSelection;
        int LastinMultipleSelection;
        bool MultipleSelectionFlag;
        bool PagingUpDown;
        int TotalLinesUsedInArea;
};
 
class  DLLAccess StringList : public List
{

	public: // used by client
		StringList();
		StringList( char const * const * StringArray );
		void SetStringList( char const * const * StringArray );
		void NowDisplaying( int const Index );
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );
	
	protected:
		char const * const * StringArray;
};



#endif //list_h_included