/****************************************************************************
 Manager.h
 
 Description:
    A field manager is a container that holds fields and allows them to
    communicate with one another.  Both Screens and Dialogs are examples 
    of field managers.  Code exists to handle focus and to allow fields to 
    exist in classes derived from FieldManager
     

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef manager_h_included
#define manager_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
class FieldManager;

#include "area.h"
#include "field.h"
#include "linkedlist.h"

typedef
enum
{
    NO_MANAGER,
    SCREEN_MANAGER,
    DIALOG_MANAGER
} MANAGERTYPE;

class DLLAccess FieldManager
{   
    public: //used by client

    	virtual void AddField( Field &newField, Field * pFieldAfterNewField=NULL );
		virtual void RemoveField( Field &newField );


        void PutFieldAtTopOfDisplay( Field * newpTopField );
		bool SetFieldwithFocus( Field * pField ); // will fail if pointer isn't in list of fields 
		Field * GetFieldwithFocus();

        Field * GetFirstField();
        Field * GetNextField();
        Field * GetNextField( Field * pField );
        Field * GetPrevField( Field * pField );



	public:    //used by UI Engine
        virtual MANAGERTYPE GetManagerType() =0;
        FieldManager();
        void Draw();
		void UnDraw();

	

		void ChangeFocusRedraw( Field & newFieldwithFocus, int const Magnitude, 
                                bool const ForwardDirection );
		void MoveFocusRedraw( int const DirectionMagnitude );

		void ClearFocus();
    
		Area& UseArea();

        virtual bool FieldManagerDisabled() = 0;
    
		Field * pFieldwithFocus;  

    protected:
		void ChangeFocus( Field & newFieldwithFocus, int const Magnitude, 
                          bool const ForwardDirection, Field * & pFieldtoDraw,
                          int &VerticalPixelInArea, FieldLocation &Location );
        void Redraw( Field & FieldtoDraw, int VerticalPixelInArea, FieldLocation Location );


		void SetFieldDisplayRect( XYRect const &RectforFields );                            
		void Reset();

		Area FieldArea;
		LinkedList ListofFields;

        bool OnLCDFlag;

        Field * pTopField;
    

};

      
#endif // manager_h_included