/*-------------------------------------------------------------------------*
 *
 * Mobitex.h
 *  Contains prototypes and structure for Mobitex specific Radio API.
 *
 * Copyright 1997-1998 Research In Motion Limited.  All Rights Reserved
 *-------------------------------------------------------------------------*/

#ifndef __RADIO_H
#define __RADIO_H

#pragma pack(push,2)

#ifdef __cplusplus
extern "C" {
#endif

#include "BaseType.h"


/*-------------------------------------------------------------------------*
 * Types.
 *-------------------------------------------------------------------------*/

// MPAK_HEADER structure
typedef struct {
    long  Sender;               // sender MAN number
    long  Destination;          // Addressee MAN number
    int   MpakType;             // PSUBCOM mpak type, 0 for non PSUBCOM
    int   HPID;                 // MPAK HPID for HPDATA Mpaks
    int   Flags;                // MPAK flags component
    TIME  Timestamp;            // parsed Mobitex Time Stamp
    long  lTime;                // MPAK raw Mobitex Time Stamp
    int   TrafficState;         // MPAK traffic state
} MPAK_HEADER;

typedef struct {
    int   RadioOn;              // RADIO_ON, RADIO_OFF
    DWORD LocalMAN;             // The modem's MAN number, as an integer
    DWORD ESN;                  // The publich ESN, not the secret one.
    int   Base;                 // MOBITEX network Base ID
    int   Area;                 // MOBITEX network Area ID
    int   RSSI;                 // RSSI_NO_COVERAGE, -113 to -40
    WORD  NetworkID;            // Mobitex network ID (0xB433 in the US)
    DWORD FaultBits;
    BOOL  Active;
    BOOL  PowerSaveMode;
    BOOL  LiveState;            // 1 = Live state, 0 = DIE state.
    BOOL  TransmitterEnabled;   // 1 = Tx enabled, 0 = Tx disabled.
} RADIO_INFO;

typedef struct {
    BYTE  SkipNum;              // Current skipnum value used by the modem
    BYTE  ProtocolRevision;     // Mobitex protocol revision (0 for pre R14N)
    BYTE  SkipTrans;            // Skip Trans value of Mobitex network
    BYTE  Mode;                 // Powersave = 1, Express = 0
} SKIPNUM_INFO;

typedef struct {
    int  DefaultNetworkIndex;   // Indicates which network is the default network
    int  CurrentNetworkIndex;   // Indicates which network is the current network
    int  NumValidNetworks;      // Indicates number of valid networks.
    struct {
        WORD NetworkId;         // Network frame sync word (0xB433 in the US)
        BYTE NetworkName[10];   // Network name.
    }Networks[10];

} NETWORKS_INFO;


/*-------------------------------------------------------------------------*
 * Function Prototypes.
 *-------------------------------------------------------------------------*/

int     __cdecl RadioCancelSendMpak( int mpakTag );
int     __cdecl RadioGetMpak( int mpakTag, MPAK_HEADER *header, BYTE *data );
int     __cdecl RadioSendMpak( MPAK_HEADER *header, BYTE *data, int length );
int     __cdecl RadioRequestSkipnum( SKIPNUM_INFO * SkipInfo, int Skipnum );

void    __cdecl RadioChangeNetworks( DWORD NetworkId, BYTE * NetworkName );
void    __cdecl RadioDeregister( void );
void    __cdecl RadioGetAvailableNetworks( NETWORKS_INFO * info );
void    __cdecl RadioGetDetailedInfo( RADIO_INFO *info );
int     __cdecl RadioGetSignalLevel( void );
int     __cdecl RadioOnOff( int mode );
void    __cdecl RadioRegister( void );
void    __cdecl RadioResumeReception( void );
void    __cdecl RadioStopReception( int mpakTag );
void    __cdecl RadioAccelerateRetries( int mpakTag );


/*-------------------------------------------------------------------------*
 * Constants.
 *-------------------------------------------------------------------------*/

#define MAX_QUEUED_MPAKS  7

/* RADIO events. */
#define  MESSAGE_RECEIVED       0x0601
#define  MESSAGE_SENT           0x0602
#define  MESSAGE_NOT_SENT       0x0603
#define  SIGNAL_LEVEL           0x0604
#define  NETWORK_STARTED        0x0605
#define  BASE_STATION_CHANGE    0x0606
#define  RADIO_TURNED_OFF       0x0607
#define  MESSAGE_STATUS         0x0608

// RADIO 'MESSAGE_NOT_SENT' error codes (in Data[0])
#define  MPAK_NOT_SENT_TXKILL        245
#define  MPAK_NOT_SENT               251
#define  MPAK_NOT_SENT_COVER         252
#define  MPAK_NOT_SENT_CANCEL        253
#define  MPAK_INCORRECT              255
#define  MPAK_INCORRECT_TYPE         256
#define  MPAK_INCORRECT_STATE        257
#define  MPAK_INCORRECT_FLAGS        258
#define  MPAK_INCORRECT_SENDLIST     259
#define  MPAK_INCORRECT_LENGTH       260
#define  MPAK_INCORRECT_ADDRESSEE    261
#define  MPAK_PERSONAL_MAN_EXIST     262  
#define  MPAK_PERSONAL_MAN_NOT_EXIST 263
#define  MPAK_NOT_SENT_DIE           265
#define  MPAK_NOT_SENT_BUFFER_FULL   266

// RADIO MESSAGE_STATUS codes
#define MPAK_TX_PENDING              1
#define MPAK_TRANSMITTING            2

// RADIO function return error codes
#define RADIO_APP_NOT_REGISTERED  -1
#define RADIO_MPAK_NOT_FOUND      -2
#define RADIO_NO_FREE_BUFFERS     -3
#define RADIO_BAD_DATA            -4
#define RADIO_BAD_TAG             -5
#define RADIO_ERROR_GENERAL       -6
#define RADIO_ILLEGAL_SKIPNUM     -7

// RadioGetSignalLevel() special value
#define  RSSI_NO_COVERAGE   -256

// RadioOnOff() modes
#define  RADIO_OFF        0
#define  RADIO_ON         1
#define  RADIO_LOWBATT    2
#define  RADIO_GET_ONOFF  3

// MPAK traffic states (in MPAK_HEADER)
#define  TS_MESSAGE_OK                  0      // MESSAGE OK
#define  TS_MESSAGE_FROM_MAILBOX        1      // MESSAGE FROM MAILBOX
#define  TS_MESSAGE_IN_MAILBOX          2      // MESSAGE WAS PLACED IN MAILBOX & COPY RETURNED
#define  TS_CANNOT_BE_REACHED           3      // RETURNED, CANNOT BE REACHED
#define  TS_ILLEGAL_MESSAGE             4      // RETURNED, BAD MESSAGE FORMAT
#define  TS_NETWORK_CONGESTED           5      // RETURNED, NETWORK CONGESTED
#define  TS_TECHNICAL_ERROR             6      // RETURNED, TECHNICAL NETWORK ERROR
#define  TS_DESTINATION_BUSY            7      // RETURNED, DESTINATION BUSY

// subscription flags (in MPAK_HEADER)
#define  FLAG_MAILBOX        1
#define  FLAG_POSACK         2
#define  FLAG_SENDLIST       4
#define  FLAG_UNKNOWN_FLAG   8

// MPAK packet types (in MPAK_HEADER)
// a value of zero will indicate that the MPAK is not of class PSUBCOM, and
// thus the user must build/parse a raw MPAK
#define  MPAK_TEXT      1
#define  MPAK_DATA      2
#define  MPAK_STATUS    3
#define  MPAK_HPDATA    4


//
// Additional definitions, not absolutely required to use the API
//

#define MAX_MPAK_SIZE       560
#define MAX_MPAK_DATA_SIZE  512

// MPAK classes
#define PCLASS_PSUBCOM  0
#define PCLASS_DTESERV  3
#define PCLASS_PSOSCOM  1  

// DTESERV-class MPAK types
#define MPAK_LOGINREQ   1
#define MPAK_LOGINGRA   2
#define MPAK_LOGINREF   3
#define MPAK_LOGOUT     4
#define MPAK_LOGOUTORD  5
#define MPAK_ACTIVE     7
#define MPAK_INACTIVE   8
#define MPAK_VICESOSRX  13
#define MPAK_SOSRX      14
#define MPAK_GROUPLIST  15
#define MPAK_FLEXLIST   17
#define MPAK_TIME       20
#define MPAK_MODE       24

// assists for parsing MPAKS
#define TRAFFIC_STATE_OFFSET 6
#define TRAFFIC_STATE_MASK   0xe0
#define TRAFFIC_STATE_SHIFT  5
#define TRAFFIC_STATE_OK     0

#define MAILBOX_FLAG_OFFSET 6
#define MAILBOX_FLAG_MASK   0x01
#define MAILBOX_FLAG_SHIFT  0

#define POSACK_FLAG_OFFSET 6
#define POSACK_FLAG_MASK   0x02
#define POSACK_FLAG_SHIFT  1

#define SENDLIST_FLAG_OFFSET 6
#define SENDLIST_FLAG_MASK   0x04
#define SENDLIST_FLAG_SHIFT  2

#define UNKNOWN_FLAG_OFFSET 6
#define UNKNOWN_FLAG_MASK   0x08
#define UNKNOWN_FLAG_SHIFT  3

#define PACKET_CLASS_OFFSET 7
#define PACKET_CLASS_MASK   0xc0
#define PACKET_CLASS_SHIFT  6

#define EXTERNAL_FLAG_OFFSET 7
#define EXTERNAL_FLAG_MASK   0x20
#define EXTERNAL_FLAG_SHIFT  5

#define PACKET_TYPE_OFFSET  7
#define PACKET_TYPE_MASK    0x1f
#define PACKET_TYPE_SHIFT   0

#define INDEPENDENT_HEADER_SIZE 8
#define TIMESTAMP_OFFSET    8
#define TIMESTAMP_LENGTH    3
#define DATA_OFFSET         11
#define SENDLIST_OFFSET     8
#define SENDLIST_LENGTH     22
#define HP_PROTOCOL_ID_OFFSET 11


#ifdef __cplusplus
}
#endif

#pragma pack(pop)

#endif  /* __RADIO_H */
