/****************************************************************************
 Screen.h
 
 Description:
    Screens are rectangular containers for a list of fields. Screens present 
    the fields in a vertical ordering, and the UI Engine is responsible for 
    displaying the screen based on the user input and the field selected by 
    the user. A screen can be composed of as many fields as possible.  The user 
    can navigate between fields, typically with the roller wheel. A screen can 
    also have a title on the first line of the display, one line in length.    
                                                                  
    
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef screen_h_included
#define screen_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

const SIX_LINE_LCD_HEIGHT = 63;

#include "warning.h"

class Screen;
#include "manager.h"
#include "uiengine.h"
#include "label.h"
#include "field.h"
#include "linkedlist.h"


class DLLAccess Screen : public FieldManager
{
    public: //used by client

		Screen();
        void ResetScreen();
	    

  		void AddLabel( Label &newTitle ); 
		void RemoveLabel( Label &TitletoRemove );

        bool IsFieldDirty();
        void MarkFieldAsClean();
        void MarkFieldAsDirty();



	public: //used by uiengine

        bool IsEmptyOfLabels();
        void ResetDimensionDependentData();
        int GetLCDParametersUsed();
        void SetLCDParametersUsed();
        MANAGERTYPE GetManagerType() { return SCREEN_MANAGER; }
        bool FieldManagerDisabled(); 
        
		virtual ~Screen();
		void DisplayScreen();
		void ClearScreen();
        void SetScreenDisabled();
		

    
		virtual RESULT HandleInput(MESSAGE const &InputMessage);

	public:
    	void SetUIEngine(UIEngine & newCallingUI);
        void ClearUIEngine();
	protected:
		UIEngine * pCallingUIEngine;
        bool DimensionsOfAreaSetPreviously;

  protected:                               
    
        int LastLCDParametersUsed;
		LinkedList ListofLabels;
        bool ScreenDisabled;
  
};

#endif