/****************************************************************************
 Separator.h
 
 Description:
    Separators are one-line fields that either have a string or bitmap on 
    them. Separators separate two other kinds of fields for readability 
    purposes and can never be selected or highlighted. By default, the 
    string associated with a separator is a line that is the length of the 
    display.     
     
 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef separator_h_included
#define separator_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"
class Separator;
#include "pager.h"
#include "field.h"

class DLLAccess Separator : public Field 
{ 
    public: //called by client 
        virtual void Redraw();

        Separator(char const * const pnewSeparatorString = NULL, int newSeparatorStringLength = -1,int const newJustification = LCD_LEFT_JUSTIFIED);
        Separator(BitMap const * const pnewSeparatorBitMap, int const newXOffset = 0);
        FIELDTYPE GetFieldType( FIELDTYPE * pDerived = NULL );


	public: //called by field manager

        void ResetDimensionDependentData(); //used for updating display parms on the fly    
        virtual void Draw(int const PixelOnScreen, FieldLocation const AnchoredFrom, int &TopPixel, int &HeightinPixels,
                   bool &MoreDataAtTop, bool &MoreDataAtBottom );
										//null for toppixel and length means the caller doesn't care
        virtual void UnDraw();
 		virtual bool GiveFocus(int const DirectionMagnitude);
		//positive direction is forward, negative is back
		//0 means leave previous focus
		virtual void TakeFocus();

		virtual int QueryLineHeight(int const MaximumRelevant);
		virtual int QueryPixelHeight(FieldLocation const StartFrom, int const MaximumRelevant);
        virtual int QueryHeightInArea( FieldLocation const StartFrom ); 
		
		virtual bool IsFocusVisible();
		
		
        virtual RESULT HandleInput(MESSAGE const &InputMessage);

    
	protected:  

        void Validate();

        bool OnLCDFlag; 

        char const * pSeparatorString;
        int SeparatorStringLength;
        BitMap const * pSeparatorBitmap;
        int JustificationOrOffset; //represents justification constant if pString is nonnull
                                    //represent offset if pBitmap is nonnull
        int CachedHeight;

    
    
};
      
#endif