/****************************************************************************
 UIEngine.h 
 
 Description:
    A client to the UIEngine should pass all input to the UIEngine class where
    it will be handled automatically by the currently active field.

 Copyright (C) 1997-1999 Research In Motion Limited
 ***************************************************************************/

#ifndef uiengine_h_included
#define uiengine_h_included

#ifdef DLLAccess
#undef DLLAccess
#endif

#ifdef _UI_ // UIEngine
#define DLLAccess DLLExport
#else
#define DLLAccess DLLImport
#endif

#include "warning.h"

class UIEngine; // class forwarding

#include "result.h"
#include "pager.h"
#include "screen.h"
#include "dialog.h"
#include "menu.h"
#include "services.h"



class DLLAccess UIEngine
{  
	public: // used by client

        UIEngine();
        ~UIEngine();
		virtual RESULT HandleInput( MESSAGE &InputMessage );
        virtual void Initialize();
 
        void ProcessScreen( Screen &newScreen );
        void ClearScreen();
        void DisableScreen();
        

        void ProcessMenu( Menu &newMenu );
        void ClearMenu();
     
        void ProcessDialog( Dialog &newDialog );
        void ClearDialog();
    
		void RestoreDisplayContext();
        int SetDisplayContext();
        void GetUIVersion( int *Major, int *Minor, int *Revision );  
        
        int ReplaceFont( int iFontIndex, const FontDefinition *pNewFont );


    public: // used by UI Engine

        void RemoveScreen( Screen &ScreentoRemove );
        void RemoveMenu( Menu &MenutoRemove );
        void RemoveDialog( Dialog &DialogtoRemove );

	protected:
    
		Screen * pCurrentScreen;
		Dialog * pCurrentDialog;
		Menu * pCurrentMenu;
      	int CurrentDisplayContext;
    
};


#endif //uiengine_h_included