//
// Copyright (C) 1999, 2000 Lyle Coder (x_coder@hotmail.com)
//

#ifndef _TAG_H_
#define _TAG_H_

#include <string.h>

class Tag
{
	friend class BrowserScreen;

public:
	Tag(char *n) : name(0), data(0), attrib(0), child(0), sibling(0), siblingTail(0), parent(0), start(0), end(0), userData(0), dataChild(0)
	{
		name = new char[strlen(n) + 1];
		strcpy(name, n);
	}

	Tag(Tag *tag) : name(0), data(0), attrib(0), child(0), sibling(0), siblingTail(0), parent(0), start(0), end(0), userData(0), dataChild(0)
	{
		name = new char[strlen(tag->name) + 1];
		strcpy(name, tag->name);

		if(tag->data) {
			data = new char[strlen(tag->data) + 1];
			strcpy(data, tag->data);
		}

		for(Attribute *a = tag->attrib; a; a = a->next) {
			if(attrib) {
				attrib->addSibling(new Attribute(a->name, a->param));
			} else {
				attrib = new Attribute(a->name, a->param);
			}
		}
	}

	//
	// Reccursively deletes all child and sibling tags... so call just once on the root node.
	//
	~Tag()
	{
		if(name) {
			delete [] name;
			name = 0;
		}
		if(data) {
			delete [] data;
			data = 0;
		}
		if(attrib) {
			delete attrib;
			attrib = 0;
		}
		if(start) {
			delete [] start;
		}
		if(end) {
			delete [] end;
		}
		if(child) {
			delete child;
			child = 0;
		}
		if(sibling) {
			delete sibling;
			sibling = 0;
		}
	}

	//
	// Add an attribute of this tag.  The new attribute will be appended to any existing 
	// attributes for this tag.  OK to delete name and param after invocation.
	//
	void addAttribute(char *name, char *param);

	//
	// Data is always in string format. Appends data to any existing data charecters.
	// OK to delete data (parameter) after invocation.
	//
	void addData(char *data, int len);

	//
	// Add a child node... if a child node exists, the tag is added as a sibling of the child node.
	// Do not delete tag (input parameter) after calling this method... the pointer is retained
	// Parameter tag is considered to be an unrelated node... ie, it cannot have any child and sibling 
	// nodes
	//
	Tag *addChild(Tag *tag);

	//
	// Returns the parent tag
	//
	Tag *endTag(void);

	Tag *getChild(void) {
		return child;
	}


	void setUserData(void *data) {
		userData = data;
	}

	void *getUserData(void) {
		return userData;
	}

	class Attribute
	{
	public:
		Attribute(char *n, char *p) : param(0), next(0)
		{
			name = new char[strlen(n) + 1];
			strcpy(name, n);
			if(p) {
				param = new char[strlen(p) + 1];
				strcpy(param, p);
			}
		}

		~Attribute()
		{
			delete [] name;

			if(param) {
				delete [] param;
			}

			//
			// Reccursively delete sibling attributes
			//
			if(next) {
				delete next;
			}
		}

		Attribute *addSibling(Attribute *attribute)
		{
			next = attribute;
			return this;
		}

	public:
		char *name;
		char *param;	
		Attribute *next;
	};

public:
	//
	// Add a sibling node.  If sibling exists, it is inserted at the head of the linked list.
	// Do not delete tag (parameter) after calling because the pointer is retained
	// Parameter tag is considered to be an unrelated node... ie, it cannot have any child and sibling 
	// nodes
	//
	Tag *addSibling(Tag *tag);

	char *name;
	char *data;
	Attribute *attrib;
	Tag *child;
	Tag *sibling;
	Tag *siblingTail;
	Tag *parent;
	char *start;
	char *end;

	//
	// Application specific data
	//
	void *userData;

private:
	Tag *dataChild;
};

#endif // _TAG_H_