//
// Copyright (C) 1999, 2000 Lyle Coder (x_coder@hotmail.com)
//

#ifndef _WAP_H_
#define _WAP_H_

#include "UI32.h"
#include "Tag.h"

//
// Constants
//
char const * const  MENU_SEPARATOR_BAR = "__";

//
// Browser Screen menu stuff
//
#define NUM_BROWSER_MENU_ITEMS 6
char const * const  browserMenuItems[NUM_BROWSER_MENU_ITEMS] = 
{
	MENU_SEPARATOR_BAR,
	"Go",
	"Follow Link",
    "Help",
	"Cancel",
    MENU_SEPARATOR_BAR
};

#define GO 1
#define FOLLOW_LINK 2
#define HELP 3
#define CANCEL 4

//
// WAPScreen is the base class for all screens used in this client 
// application
//
class WAPScreen: public Screen 
{
public:
        
	WAPScreen(UIEngine& uiEngine, char const * const * const ppMenuItems, int const nNumMenuItems) : m_menu(ppMenuItems, nNumMenuItems), m_uiEngine(uiEngine) {}

	// A function that will display the menu for the screen
    // and return the index of the selected item.
    virtual int HandleMenu() = 0;
	virtual void Display() = 0;

protected:

	// The menu for the screen.
	Menu m_menu;

	// The UIEngine object to use when processing the screen
    UIEngine m_uiEngine;
};

//
// The browser can display clipped web sites.  This also hooks in with the 
// smart web search results.
//
class BrowserScreen: public WAPScreen 
{
	//
	// Need C functions because the expat parser is in C
	//
	friend void start_element(void *user_data, const char *name, const char **attr);
	friend void end_element(void *user_data, const char *name);
	friend void data_element(void *user_data, const char *name, int len);

public:

	BrowserScreen(UIEngine& uiEngine) : current(0), active(0), WAPScreen(uiEngine, browserMenuItems, NUM_BROWSER_MENU_ITEMS)
	{
		// Add the title to the screen.
        AddLabel(m_title);
		AddField(m_url);

        // Set up the text in the title bar
        m_title.SetText("WAP Browser");

		m_url.SetLabel("URL:");
		m_url.SetBuffer(16, 32);
		m_url.SetProperties(Edit::CR_TO_ROLL_DOWN, 0);
		m_url.SetProperties(CTYPE::URL, 0);
	}

	//
	// Inherrited functions
	//
	int HandleMenu();

	//
	// Show the browser screen and handle input
	//
    void Display();

	//
	// Will parse the WML content passed in and represent it as a Tag tree rooted at current
	//
	void Parse(char *wml);

	//
	// Will walk the Tag tree (rooted at 'root' param) and show the WML content
	//
	void Print(Tag *root);

	//
	// Walk the tree and delete all user data (Edit fields)
	//
	void Delete(Tag *root);


	//
	// Set the tag the user is scrolling on
	//
	void setActiveTag(Tag *active) {
		this->active = active;
	}

protected:

	//
    // UI components for the browser screen
	//
    Title m_title;
	Edit m_url;
	char m_urlText[40];

	//
	// The DOM (Tag tree) root node for the WML content currently being parsed and displayed
	//
	Tag *current;

	Tag *active;

	//
	// Hold the last parse error message
	//
	char error[128];

	//
	// Hold the WML content from multiple MPAK messages
	//
	char page[2048];

#if 0
	char* find(char* haystack, char* needle)
	{
		unsigned int i, j;

		for(i = 0; i < (2048 - strlen(needle)); i++) {
			int found = 1;
			for(j = 0; j < strlen(needle); j++) {
				if(*(haystack + i + j) != *(needle + j)) {
					found = 0;
				}
			}
			if(found) {
				return haystack + i;
			}
		}
		return 0;
	}
#endif
	int strncasecmp(const char *s1, const char *s2, int n)
	{
		unsigned char c1, c2;
		int i = 0;

		if (s1 == s2) {
			return 0;
		}

		do {
			c1 = tolower(*s1++);
			c2 = tolower(*s2++);
			if(c1 == '\0') {
				break;
			}
		} while((c1 == c2) && (++i < n));

		return c1 - c2;
	}

	//
    // FIXME This is a horrible performance bug!!! 
    // But hey I'm hackin this thing just yet to see if it works! :)
    //
	char *find(char *haystack, char *needle)
	{
		for(int i = 0; i < (int) strlen(haystack); i++) {
			if(!strncasecmp(&haystack[i], needle, strlen(needle))) {
				return &haystack[i];
			}
		}
		return 0;
	}
	
};

#endif // _WAP_H_